'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Formset = function () {
    function Formset(selector, options) {
        _classCallCheck(this, Formset);

        this.selector = selector;
        this.options = options;
        this.flatExtraClasses = options.extraClasses.join(' ');
        this.deleteButtonEvents();
        if (selector.length) {
            this.addButtonEvent();
        }
    }

    _createClass(Formset, [{
        key: 'deleteButtonEvents',
        value: function deleteButtonEvents() {
            var _this = this;

            this.selector.each(function (i, el) {
                var formsetElement = $(el);
                _this.replaceCheckboxes(formsetElement);
                if (_this.hasChildElements(formsetElement)) {
                    // Add click events to the delete buttons.
                    formsetElement.find('a.' + _this.options.deleteCssClass).click(function (e) {
                        var _row = $(e.currentTarget).closest('.' + _this.options.formCssClass);
                        var _del = _row.find('input:hidden[id $= "-DELETE"]');

                        // If a pre-delete callback was provided, call it with the formsetElement and all forms:
                        // To prevent deletion return false
                        if (_this.options.beforeDelete) {
                            if (_this.options.beforeDelete(_del, _row, $('.' + _this.options.formCssClass + ':visible'))) {
                                _this.deleteRow(_del, _row);
                            }
                        } else {
                            _this.deleteRow(_del, _row);
                        }
                        if (_this.options.keepFirst) {
                            if ($('.' + _this.options.formCssClass + ':visible').length === 1) {
                                _this.disableFirstDeleteLink();
                            }
                        }
                        return false;
                    });

                    formsetElement.addClass(_this.options.formCssClass);
                    _this.applyExtraClasses(formsetElement, i);
                }
            });
        }
    }, {
        key: 'replaceCheckboxes',
        value: function replaceCheckboxes(formsetElement) {
            var del = formsetElement.find('input:checkbox[id $= "-DELETE"]');
            // Replace default formset checkboxes with hidden fields.
            if (del.length) {
                del.before('<input type="hidden" name="' + del.attr('name') + '" id="' + del.attr('id') + '" />');
                del.remove();
            }
        }
    }, {
        key: 'enableDeleteLinks',
        value: function enableDeleteLinks() {
            $('.' + this.options.formCssClass).find('.delete-row').removeClass('disabled');
        }
    }, {
        key: 'disableFirstDeleteLink',
        value: function disableFirstDeleteLink() {
            $('.' + this.options.formCssClass + ':visible:first').find('.' + this.options.deleteCssClass).addClass('disabled');
        }

        /**
         * Make a clone of the last formset and return it.
         */

    }, {
        key: 'cloneFormset',
        value: function cloneFormset() {
            var template = $('.' + this.options.formCssClass + ':last').clone(true).removeAttr('id');
            template.find('input:hidden[id $= "-DELETE"]').remove();
            template.find('.' + this.options.errorCssClass).remove();
            template.find('input,select,textarea,label').each(function (i, el) {
                // If this is a checkbox or radiobutton, uncheck it.
                // This fixes Issue 1, reported by Wilson.Andrew.J:
                if ($(el).is('input:checkbox') || $(el).is('input:radio')) {
                    $(el).attr('checked', false);
                } else {
                    $(el).val('');
                }
            });
            return template;
        }
    }, {
        key: 'addButtonEvent',
        value: function addButtonEvent() {
            var _this2 = this;

            var addButton = void 0;

            this.options.formTemplate = this.cloneFormset();
            // Hide the first formset on init, when there is only one formset and
            // keepFirst is true.
            if (this.options.keepFirst && $('.' + this.options.formCssClass + ':visible').length === 1) {
                this.disableFirstDeleteLink();
            }

            if (this.selector.prop('tagName') === 'TR') {
                // Insert the `add` button in a new table row if forms are laid out as table rows.
                var numCols = this.selector.eq(0).children().length;
                this.selector.parent().append('<tr><td class="django-formset-add td" colspan="' + numCols + '"><a class="' + this.options.addCssClass + '" href="javascript:void(0)">' + this.options.addText + '</a></tr>');
                addButton = this.selector.parent().find('tr:last a');
                addButton.parents('tr').addClass(this.options.formCssClass + '-add');
            } else {
                // Otherwise, insert it immediately after the last form in a div.
                this.selector.filter(':last').after('<div class="django-formset-add div"><a class="' + this.options.addCssClass + '" href="javascript:void(0)">' + this.options.addText + '</a></div>');
                addButton = this.selector.filter(':last').next();
            }

            addButton.click(function (e) {
                var totalForms = parseInt($('#id_' + _this2.options.prefix + '-TOTAL_FORMS').val(), 10);
                var formsetElement = _this2.options.formTemplate.clone(true);
                var buttonRow = $(e.currentTarget).parents('tr.' + _this2.options.formCssClass + '-add').get(0) || e.currentTarget;
                var forms = $('.' + _this2.options.formCssClass);
                // If a pre-add callback was provided, call it with the row and all forms:
                // To prevent adding return false
                if (_this2.options.beforeAdd) {
                    if (_this2.options.beforeAdd(formsetElement, $('.' + _this2.options.formCssClass + ':visible').length, buttonRow, forms)) {
                        _this2.addRow(formsetElement, totalForms, buttonRow);
                    }
                } else {
                    _this2.addRow(formsetElement, totalForms, buttonRow);
                }

                // Check if the first formset should have a delete button again.
                // Unhide all, because inline styling is copied for new formsets.
                if (_this2.options.keepFirst && $('.' + _this2.options.formCssClass + ':visible').length > 1) {
                    _this2.enableDeleteLinks();
                }
                return false;
            });
        }
    }, {
        key: 'applyExtraClasses',
        value: function applyExtraClasses(formsetElement, ndx) {
            if (this.options.extraClasses) {
                formsetElement.removeClass(this.flatExtraClasses);
                formsetElement.addClass(this.options.extraClasses[ndx % this.options.extraClasses.length]);
            }
        }
    }, {
        key: 'updateElementIndex',
        value: function updateElementIndex(elem, prefix, ndx) {
            var idRegex = new RegExp('(' + prefix + '-\\d+-)|(^)');
            var replacement = prefix + '-' + ndx + '-';
            if (elem.attr('for')) {
                elem.attr('for', elem.attr('for').replace(idRegex, replacement));
            }
            if (elem.attr('id')) {
                elem.attr('id', elem.attr('id').replace(idRegex, replacement));
            }
            if (elem.attr('name')) {
                elem.attr('name', elem.attr('name').replace(idRegex, replacement));
            }
        }
    }, {
        key: 'hasChildElements',
        value: function hasChildElements(formsetElement) {
            return formsetElement.find('input,select,textarea,label').length > 0;
        }
    }, {
        key: 'addRow',
        value: function addRow(formsetElement, formCount, buttonRow) {
            var _this3 = this;

            this.applyExtraClasses(formsetElement, formCount);
            formsetElement.insertBefore($(buttonRow)).show();
            formsetElement.find('input,select,textarea,label').each(function (i, el) {
                _this3.updateElementIndex($(el), _this3.options.prefix, formCount);
            });
            $('#id_' + this.options.prefix + '-TOTAL_FORMS').val(formCount + 1);
            // If a post-add callback was supplied, call it with the added form:
            if (this.options.added) this.options.added(formsetElement);
        }
    }, {
        key: 'deleteRow',
        value: function deleteRow(del, formsetElement) {
            var _this4 = this;

            if (del.length) {
                // We're dealing with an inline formset; rather than remove
                // this form from the DOM, we'll mark it as deleted and hide
                // it, then let Django handle the deleting:
                del.val('on');
                formsetElement.hide();
            } else {
                formsetElement.remove();
                var forms = $('.' + this.options.formCssClass);
                // Update the TOTAL_FORMS form count.
                // Also update names and IDs for all remaining form controls so they remain in sequence:
                $('#id_' + this.options.prefix + '-TOTAL_FORMS').val(forms.length);

                var _loop = function _loop(i, formCount) {
                    _this4.applyExtraClasses(forms.eq(i), i);
                    forms.eq(i).find('input,select,textarea,label').each(function (_i, el) {
                        _this4.updateElementIndex($(el), _this4.options.prefix, i);
                    });
                };

                for (var i = 0, formCount = forms.length; i < formCount; i++) {
                    _loop(i, formCount);
                }
            }
            // If a post-delete callback was provided, call it with the deleted form:
            if (this.options.deleted) this.options.deleted(formsetElement);
        }
    }]);

    return Formset;
}();

$.fn.formset = function (opts) {
    var selector = $(this);
    new Formset(selector, $.extend({}, $.fn.formset.defaults, opts));
    return selector;
};

/* Setup plugin defaults */
$.fn.formset.defaults = {
    prefix: 'form', // The form prefix for your django formset
    formTemplate: null, // The jQuery selection cloned to generate new form instances
    addText: '<i class="icon-plus-sign icon-large"></i>',
    deleteText: '<i class="icon-remove-sign icon-large"></i>',
    addCssClass: 'add-row', // CSS class applied to the add link
    deleteCssClass: 'delete-row', // CSS class applied to the delete link
    formCssClass: 'dynamic-form', // CSS class applied to each form in a formset
    extraClasses: [], // Additional CSS classes, which will be applied to each form in turn
    beforeAdd: null, // Function called each time before a new form is added
    beforeDelete: null, // Function called each time before a form is deleted
    added: null, // Function called each time a new form is added
    deleted: null, // Function called each time a form is deleted
    errorCssClass: 'errorlist' };